package org.ow2.orchestra.definition.activity;

import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jbpm.pvm.internal.model.ObservableElementImpl;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.handlers.FinishedInstanceHandler;
import org.ow2.orchestra.util.EnvTool;

public class Process extends Scope {

  private static final Logger LOG = Logger.getLogger(Process.class.getName());

  protected Process() {
    super();
  }
  public Process(final String uuid) {
    super(uuid);
    this.setName("Process " + uuid);
  }

  /**
   * Process is not an activity so we do perform hook.
   */
  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {
  }

  /**
   * Process is not an activity so we do perform hook.
   */
  @Override
  public void afterRunned(final BpelExecution bpelExecution) {
  }

  @Override
  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      super.signal(execution, signal, parameters);
      return;
    }
    // execute source to log end of scope ?
    executeSources(execution);
    if (execution.isActive()) {
      EnvTool.getRecorder().recordInstanceEnded(execution.getProcessInstanceData());
      // Call the FinishedInstanceHandler
      FinishedInstanceHandler handler = EnvTool.get(FinishedInstanceHandler.class, FinishedInstanceHandler.DEFAULT_KEY);
      handler.handleFinishedInstance(execution.getProcessInstanceData());

      final ObservableElementImpl parent = execution.getNode().getParent();
      if (parent == null) {
        if (LOG.isLoggable(Level.FINE)) {
          LOG.fine("top level scope ends the bpel process execution");
        }
        if (execution.getParent() != null) {
          throw new IllegalStateException("Child execution reaches the end of process !");
        }
        if (execution.getWaitingExecutions() != null && !execution.getWaitingExecutions().isEmpty()) {
          throw new IllegalStateException(
              "Execution reaches the end of process, but child executions are still waiting for a trigger !");
        }
        execution.end();
      }
    }
  }
}
