package org.ow2.orchestra.env;

import java.io.InputStream;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;

import org.jbpm.pvm.env.EnvironmentFactory;
import org.jbpm.pvm.internal.env.PvmEnvironmentFactoryParser;
import org.jbpm.pvm.internal.xml.Bindings;
import org.jbpm.pvm.internal.xml.Parse;
import org.ow2.orchestra.env.binding.ChainerBinding;
import org.ow2.orchestra.env.binding.HibernateBpelPersistenceServiceBinding;
import org.ow2.orchestra.env.binding.ImplementationBinding;
import org.ow2.orchestra.env.binding.QuerierDbSessionBinding;
import org.ow2.orchestra.env.binding.QuerierListAccessorBinding;
import org.xml.sax.InputSource;

public class BpelEnvironmentParser extends PvmEnvironmentFactoryParser {

  private static PvmEnvironmentFactoryParser instance;

  public static final String CLASSPATH_URL = "classpath://";

  public static synchronized PvmEnvironmentFactoryParser getInstance() {
    if (instance == null) {
      instance = new BpelEnvironmentParser();
      // Get bindings
      final Bindings bindings = instance.getEnvironmentXmlParser().getBindings();
      // Use the same bindings in block and application contexts
      instance.getEnvironmentFactoryXmlParser().setBindings(bindings);
      // Add orchestra bindings
      bindings.addBinding(new HibernateBpelPersistenceServiceBinding());
      bindings.addBinding(new QuerierDbSessionBinding());
      bindings.addBinding(new ImplementationBinding("repository"));
      bindings.addBinding(new ImplementationBinding("invoker"));
      bindings.addBinding(new ImplementationBinding("publisher"));
      bindings.addBinding(new ImplementationBinding("recorder"));
      bindings.addBinding(new ImplementationBinding("journal"));
      bindings.addBinding(new ImplementationBinding("finished-instance-handler"));
      bindings.addBinding(new ChainerBinding());
      bindings.addBinding(new ImplementationBinding("history"));
      bindings.addBinding(new ImplementationBinding("archiver"));
      bindings.addBinding(new QuerierListAccessorBinding());
    }
    return instance;
  }

  public static EnvironmentFactory parseEnvironmentFactoryFromXmlString(
      final String xmlString) {
    final Parse parse = getInstance().createParse();
    final EnvironmentFactory factory = (EnvironmentFactory) parse.setString(xmlString).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  public static synchronized EnvironmentFactory parseEnvironmentFactoryFromResource(
      final String resource) {
    final Parse parse = getInstance().createParse();
    final EnvironmentFactory factory = (EnvironmentFactory) parse.setResource(resource).execute().getDocumentObject();
    parse.checkProblems("environment");
    return factory;
  }

  @Override
  public synchronized DocumentBuilderFactory getDocumentBuilderFactory() {
    this.documentBuilderFactory = newDocumentBuilderFactory();
    this.documentBuilderFactory.setNamespaceAware(true);
    this.documentBuilderFactory.setValidating(true);
    // ignore white space can only be set if parser is validating
    this.documentBuilderFactory.setIgnoringElementContentWhitespace(true);
    // select xml schema as the schema language (a.o.t. DTD)
    this.documentBuilderFactory.setAttribute(
        "http://java.sun.com/xml/jaxp/properties/schemaLanguage",
    "http://www.w3.org/2001/XMLSchema");
    //  set schema sources
    final URL url = Thread.currentThread().getContextClassLoader().getResource("environment-orchestra.xsd");
    this.documentBuilderFactory.setAttribute(
        "http://java.sun.com/xml/jaxp/properties/schemaSource", url.toExternalForm());
    return this.documentBuilderFactory;
  }

  /* (non-Javadoc)
   * @see org.jbpm.xml.Parser#resolveEntity(java.lang.String, java.lang.String)
   */
  @Override
  public InputSource resolveEntity(final String publicId, final String systemId) {
    if (systemId.startsWith(CLASSPATH_URL)) {
      final String localpart = systemId.substring(CLASSPATH_URL.length());
      final InputStream inStream = Thread.currentThread().getContextClassLoader().getResourceAsStream(localpart);
      if (inStream != null) {
        return new InputSource(inStream);
      }
    }
    return super.resolveEntity(publicId, systemId);
  }
}
