/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.jmx;

import java.lang.reflect.Proxy;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.data.ProcessDefinitionData;
import org.ow2.orchestra.facade.exception.InstanceNotFoundException;
import org.ow2.orchestra.facade.exception.ProcessNotFoundException;
import org.ow2.orchestra.facade.jmx.Deployment;
import org.ow2.orchestra.facade.jmx.InstanceInfo;
import org.ow2.orchestra.facade.jmx.RemoteDeployerMBean;
import org.ow2.orchestra.facade.runtime.ActivityInstance;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.runtime.ProcessState;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.jmx.ClassLoaderInterceptor;
import org.ow2.orchestra.jmx.JMXAgent;
import org.ow2.orchestra.jmx.RemoteDeployerImpl;

public class RemoteDeployer implements RemoteDeployerMBean {

  private final RemoteDeployerMBean impl;

  public RemoteDeployer() {
    final RemoteDeployerImpl remoteDeployer = new RemoteDeployerImpl();
    final Class<RemoteDeployerMBean> clazz = RemoteDeployerMBean.class;
    final ClassLoaderInterceptor clInterceptor = new ClassLoaderInterceptor(remoteDeployer);
    this.impl = (RemoteDeployerMBean) Proxy.newProxyInstance(JMXAgent.class.getClassLoader(), new Class[] {
      clazz
    }, clInterceptor);
  }

  public ProcessDefinitionData deploy(final Deployment deployment) {
    return this.impl.deploy(deployment);
  }

  public ProcessDefinitionData deployBar(final byte[] barFile) {
    return this.impl.deployBar(barFile);
  }

  public void initialize() {
    this.impl.initialize();
  }

  public List<ProcessDefinitionData> listProcess() {
    return this.impl.listProcess();
  }

  public boolean undeploy(final QName processQName) {
    return this.impl.undeploy(processQName);
  }

  public Set<ActivityInstance> findActivityInstances(final ProcessInstanceUUID processUUID) throws InstanceNotFoundException {
    return this.impl.findActivityInstances(processUUID);
  }

  public Set<ActivityInstance> findActivityInstances(final ProcessInstanceUUID processUUID, final ActivityState activityState)
      throws InstanceNotFoundException {
    return this.impl.findActivityInstances(processUUID, activityState);
  }

  public Set<ProcessInstance> findProcessInstances() {
    return this.impl.findProcessInstances();
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID)
      throws ProcessNotFoundException {
    return this.impl.findProcessInstances(processDefinitionUUID);
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessDefinitionUUID processDefinitionUUID,
      final ProcessState processState) throws ProcessNotFoundException {
    return this.impl.findProcessInstances(processDefinitionUUID, processState);
  }

  public Set<ProcessInstance> findProcessInstances(final ProcessState processState) {
    return this.impl.findProcessInstances(processState);
  }

  public Map<QName, List<InstanceInfo>> listInstance() {
    return this.impl.listInstance();
  }

  public Map<QName, List<String>> listInstanceStatus() {
    return this.impl.listInstanceStatus();
  }

}
