/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.runtime.impl;

import java.util.HashSet;
import java.util.Set;

import org.ow2.orchestra.facade.runtime.ActivityFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class ActivityFullInstanceImpl extends ActivityInstanceImpl implements ActivityFullInstance {

  private static final long serialVersionUID = 8901738568448948365L;

  protected Set<ActivityFullInstance> enclosedActivities;

  // for persistence
  protected long dbid;

  protected ActivityFullInstanceImpl() {
  }

  public ActivityFullInstanceImpl(final ActivityInstanceUUID uuid, final ProcessDefinitionUUID processUUID,
      final ProcessInstanceUUID instanceUUID) {
    super(uuid, processUUID, instanceUUID);
    this.enclosedActivities = new HashSet<ActivityFullInstance>();
  }

  public ActivityFullInstanceImpl(final ActivityFullInstance activityFullInstance) {
    super(activityFullInstance);
    this.enclosedActivities = new HashSet<ActivityFullInstance>();
    for (ActivityFullInstance enclosedActivityFullInstance : activityFullInstance.getEnclosedActivities()) {
      enclosedActivities.add(new ActivityFullInstanceImpl(enclosedActivityFullInstance));
    }
  }

  public Set<ActivityFullInstance> getEnclosedActivities() {
    return this.enclosedActivities;
  }

  public void addEnclosedActivity(final ActivityFullInstance enclosedActivity) {
    this.enclosedActivities.add(enclosedActivity);
    getEnclosedActivitiesUUID().add(enclosedActivity.getUUID());
  }

  @Override
  public Set<ActivityInstanceUUID> getEnclosedActivitiesUUID() {
    if (enclosedActivitiesUUID == null) {
      Set<ActivityInstanceUUID> enclosedActivitiesUUID = new HashSet<ActivityInstanceUUID>();
      for (ActivityFullInstance enclosedActivity : enclosedActivities) {
        enclosedActivitiesUUID.add(enclosedActivity.getUUID());
      }
      this.enclosedActivitiesUUID = enclosedActivitiesUUID;
    }
    return enclosedActivitiesUUID;
  }

  @Override
  public int hashCode() {
    return super.hashCode();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    if (super.equals(obj)) {
      ActivityFullInstanceImpl other = (ActivityFullInstanceImpl) obj;
      return enclosedActivities.equals(other.enclosedActivities);
    } else {
      return false;
    }
  }

}
