/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.impl;

import org.ow2.orchestra.facade.runtime.ProcessInstance;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class ProcessInstanceImpl extends RuntimeRecordImpl implements ProcessInstance {

  private static final long serialVersionUID = 6042753699327821410L;

  /**
   * Warning this field is not persisted so you have to use getter
   * instead of direct reference.
   */
  protected ActivityInstanceUUID enclosedActivityUUID;

  @Override
  public String toString() {
    return this.getClass().getName()
      + "[uuid: " + getUUID()
      + ", processDefinitionUUID: " + getProcessDefinitionUUID()
      + ", startedDate: " + getStartedDate()
      + ", endedDate: " + getEndedDate()
      + "]\n";
  }

  protected ProcessInstanceImpl() {
  }

  protected ProcessInstanceImpl(final ProcessInstanceUUID uuid, final ProcessDefinitionUUID processUUID) {
    super(processUUID, uuid);
    enclosedActivityUUID = null;
  }

  public ProcessInstanceImpl(final ProcessInstance record) {
    super(record);
    enclosedActivityUUID = record.getEnclosedActivityUUID();
  }

  public ProcessInstanceUUID getUUID() {
    return getProcessInstanceUUID();
  }

  public void setEnclosedActivityUUID(final ActivityInstanceUUID enclosedActivitiesUUID) {
    this.enclosedActivityUUID = enclosedActivitiesUUID;
  }

  public ActivityInstanceUUID getEnclosedActivityUUID() {
    return enclosedActivityUUID;
  }

  @Override
  public int hashCode() {
    return super.hashCode();
  }

  @Override
  public boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    if (!super.equals(obj)) {
      return false;
    }
    ProcessInstanceImpl other = (ProcessInstanceImpl) obj;
    ActivityInstanceUUID activityInstanceUUID = getEnclosedActivityUUID();
    ActivityInstanceUUID otherActivityInstanceUUID = other.getEnclosedActivityUUID();
    if (activityInstanceUUID == null) {
      if (otherActivityInstanceUUID != null) {
        return false;
      }
    } else {
      if (!activityInstanceUUID.equals(otherActivityInstanceUUID)) {
        return false;
      }
    }
    return true;
  }

}
